/*********************************************************************/
/* hex.c     v 2.3      by J.Sobczyk            2001.08.02  17:01    */
/*                                                                   */
/* Polish document converter.                                        */
/* Simple hex dump.                                                  */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "polcnv.h"

long int hex_hits  = 0;
long int hex_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void hex_decode( void )
  {
  int limit;

  limit = inp_ffree;
  if (!inp_eof)
    while (limit > inp_fused)
      {
      if (inp_buf[limit-1] == ' ')
	break;
      limit--;
      }

  if (limit == inp_fused)
    {
    fprintf( stderr, "Not a hex file\n" );
    exit( 1 );
    }

  while ((inp_fused < limit) &&
	 (uni_ffree < uni_size ))
    {
    while ((inp_fused < limit)  && isspace(inp_buf[inp_fused]))
      inp_fused++;
    if (inp_fused == limit)
      break;
    uni_buf[ uni_ffree ] = 0;
    while ((inp_fused < limit)  && isxdigit(inp_buf[inp_fused]))
      {
      uni_buf[ uni_ffree ] <<= 4;
      if (isdigit(inp_buf[inp_fused]))
        uni_buf[ uni_ffree ] += inp_buf[inp_fused++] - '0';
      else if (isupper(inp_buf[inp_fused]))
        uni_buf[ uni_ffree ] += inp_buf[inp_fused++] - 'A' + 10;
      else 
        uni_buf[ uni_ffree ] += inp_buf[inp_fused++] - 'a' + 10;
      }
    uni_ffree++;
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void hex_encode( void )
  {
  static int line_len = 0;

  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size-10))
    {
    sprintf( (char *)&out_buf[out_ffree], "%08lX ", uni_buf[uni_fused] );

    line_len  += strlen( (char *) &out_buf[out_ffree] );
    out_ffree += strlen( (char *) &out_buf[out_ffree] );
    if (line_len > 71)
      {
      out_buf[out_ffree++] = '\n';
      line_len = 0;
      }

    uni_fused++;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    {
    out_buf[out_ffree++] = '\n';
    out_eof = TRUE;
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_hex( byte *buff )
  {
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_hex( void )
  {
  return hex_hits-hex_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_hex( void )
  {
  trans[ TYPE_HEX ].key			= (char*)"-x";
  trans[ TYPE_HEX ].long_key		= (char*)"-hex";
  trans[ TYPE_HEX ].name		= (char*)"simple UCS-4 hex dump";
  trans[ TYPE_HEX ].reset_inp		= NULL;
  trans[ TYPE_HEX ].reset_out		= NULL;
  trans[ TYPE_HEX ].decode		= hex_decode;
  trans[ TYPE_HEX ].encode		= hex_encode;
  trans[ TYPE_HEX ].inp_tbl		= NULL;
  trans[ TYPE_HEX ].inp_tbl_size	= 0;
  trans[ TYPE_HEX ].out_tbl		= NULL;
  trans[ TYPE_HEX ].out_tbl_size	= 0;
  trans[ TYPE_HEX ].buf_tst		= NULL;
  trans[ TYPE_HEX ].freq_tst		= test_freq_hex;
  }

/*********************************************************************/
