/*********************************************************************/
/* ohida.C      v 2.5      by J.Sobczyk            2002.05.02  23:01 */
/*                                                                   */
/* Polish document converter.                                        */
/* TeX ohida                                                         */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
typedef struct
  {
    char *str;
    unicode chr;
  } ohida_r;
/*-------------------------------------------------------------------*/
ohida_r ohida_tbl[2*POL_NR] = /* TeX ohida */
  {		
    { "\\k{a}",	0x0105 },   /* LATIN SMALL LETTER A WITH OGONEK */
    { "\\'c",	0x0107 },   /* LATIN SMALL LETTER C WITH ACUTE */
    { "\\k{e}",	0x0119 },   /* LATIN SMALL LETTER E WITH OGONEK */
    { "\\l{}",	0x0142 },   /* LATIN SMALL LETTER L WITH STROKE */
    { "\\'n",	0x0144 },   /* LATIN SMALL LETTER N WITH ACUTE */
    { "\\'o",	0x00F3 },   /* LATIN SMALL LETTER O WITH ACUTE */
    { "\\'s",	0x015B },   /* LATIN SMALL LETTER S WITH ACUTE */
    { "\\'z",	0x017A },   /* LATIN SMALL LETTER Z WITH ACUTE */
    { "\\.z",	0x017C },   /* LATIN SMALL LETTER Z WITH DOT ABOVE */
    { "\\k{A}",	0x0104 },   /* LATIN CAPITAL LETTER A WITH OGONEK */
    { "\\'C",	0x0106 },   /* LATIN CAPITAL LETTER C WITH ACUTE */
    { "\\k{E}",	0x0118 },   /* LATIN CAPITAL LETTER E WITH OGONEK */
    { "\\L{}",	0x0141 },   /* LATIN CAPITAL LETTER L WITH STROKE */
    { "\\'N",	0x0143 },   /* LATIN CAPITAL LETTER N WITH ACUTE */
    { "\\'O",	0x00D3 },   /* LATIN CAPITAL LETTER O WITH ACUTE */
    { "\\'S",	0x015A },   /* LATIN CAPITAL LETTER S WITH ACUTE */
    { "\\'Z",	0x0179 },   /* LATIN CAPITAL LETTER Z WITH ACUTE */
    { "\\.Z",	0x017B }    /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

long int ohida_hits  = 0;
long int ohida_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ohida_decode( void )
  {
  int i, j;
  unicode ch0;
  unicode ch1;
  int limit;
 
  limit = inp_ffree;
  if (!inp_eof)
    limit--;
 
  while ((inp_fused < limit) &&
         (uni_ffree < uni_size ))
    {
    ch0 = (unicode) inp_buf[ inp_fused++ ];

    if      (ch0 == '\\')
      {
      for (i=0; i< 2*POL_NR; i++)
        {
        for (j=1; j<strlen(ohida_tbl[i].str); j++)
	  if (inp_buf[ inp_fused+j-1 ] != ohida_tbl[i].str[j])
	    break;
	if (j == strlen(ohida_tbl[i].str))
	  {
	  inp_fused+=j-1;
	  ch0 = ohida_tbl[i].chr;
	  break;
	  }
	}
      }
    uni_buf[ uni_ffree++ ] = ch0;
    }
 
  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ohida_encode( void )
  {
  int i, j;
  unicode ch;
 
  while ((uni_ffree > uni_fused) &&
         (out_ffree < out_size-1))
    {
    ch = uni_buf[uni_fused++];
 
    for (i=0; i< 2*POL_NR; i++)
      if (ch == ohida_tbl[i].chr)
	{
	for (j=0; j<strlen(ohida_tbl[i].str); j++)
	  {
	  ch = ohida_tbl[i].str[j];
	  out_buf[out_ffree++] = (byte) ch;
	  }
	out_ffree--;
	break;
	}
 
    out_buf[out_ffree++] = (byte) ch;
    }
 
  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_ohida( byte *buff )
  {
  int i;

  /*
  for (i=0; i< 2; i++)
    if (buff[0] == ohida_tbx[i][0])
      {
      ohida_hits++;
      return;
      }
      
  for (i=0; i< 2*POL_NR-2; i++)
    if (((byte) buff[0] == ohida_tbl[i][0]) &&
        ((byte) buff[1] == ohida_tbl[i][1])   )
      {
      ohida_hits++;
      return;
      }

  if (buff[0] & 0x80)
    ohida_fails++;
  */
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_ohida( void )
  {
  return ohida_hits-ohida_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_ohida( void )
  {
  trans[ TYPE_OHIDA ].key		= (char *)"-o";
  trans[ TYPE_OHIDA ].long_key		= (char *)"-ohida";
  trans[ TYPE_OHIDA ].name		= (char *)"TeX ohida";
  trans[ TYPE_OHIDA ].reset_inp		= NULL;;
  trans[ TYPE_OHIDA ].reset_out		= NULL;;
  trans[ TYPE_OHIDA ].decode		= ohida_decode;
  trans[ TYPE_OHIDA ].encode		= ohida_encode;
  trans[ TYPE_OHIDA ].inp_tbl		= NULL;
  trans[ TYPE_OHIDA ].inp_tbl_size	= 0;
  trans[ TYPE_OHIDA ].out_tbl		= NULL;
  trans[ TYPE_OHIDA ].out_tbl_size	= 0;
  trans[ TYPE_OHIDA ].buf_tst		= NULL;
  trans[ TYPE_OHIDA ].freq_tst		= test_freq_ohida;
  }

/*********************************************************************/
