/*********************************************************************/
/* polcnv.h     v 2.3      by J.Sobczyk            2001.08.01  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include "../config.h"

#define TRUE		1
#define FALSE		0

#define POL_NR		9

/*#define BUF_SIZE	512*/
#ifdef DOS
#define BUF_SIZE	16384-40
#else
#define BUF_SIZE	32768
#endif

#define BUF_ZONE	32

typedef unsigned char byte;
typedef unsigned long unicode;

typedef unicode (*tbl_ptr)[2];
typedef unicode (code_tbl)[2*POL_NR][2];

/*-------------------------------------------------------------------*/
extern int  inp_type;
extern byte inp_prefix;
extern char *inp_name;
extern FILE *inp_file;
extern tbl_ptr inp_tbl;
extern int  inp_tbl_size;

extern int  inp_ffree;
extern int  inp_fused;
extern int  inp_size;
extern int  inp_eof;
/*extern byte inp_buf[ BUF_SIZE + BUF_ZONE ];*/
extern byte *inp_buf;
extern void (*decode)( void );

/*-------------------------------------------------------------------*/
extern int  uni_ffree;
extern int  uni_fused;
extern int  uni_size;
extern int  uni_eof;
/*extern unicode uni_buf[ BUF_SIZE + BUF_ZONE ];*/
extern unicode *uni_buf;

/*-------------------------------------------------------------------*/
extern int  out_type;
extern byte out_prefix;
extern char *out_name;
extern FILE *out_file;
extern tbl_ptr out_tbl;
extern int  out_tbl_size;
extern void (*encode)( void );

extern int  out_ffree;
extern int  out_fused;
extern int  out_size;
extern int  out_eof;
/*extern byte out_buf[ BUF_SIZE + BUF_ZONE ];*/
extern byte *out_buf;

/*-------------------------------------------------------------------*/
extern void help( void );
extern void table_decode( void );
extern void table_encode( void );
extern int  table_test( tbl_ptr table, int table_size );
extern void shift( void );
extern void readfile( void );

extern int  verbose_flag;

/*===================================================================*/
#define		TYPE_UNSPECIFIED	-1 
#define		TYPE_TEST      		0
#define		TYPE_PREFIX		1
#define		TYPE_FILE      		2
#define		TYPE_ISO_8859_2		3
#define		TYPE_MAZOWIA		4
#define		TYPE_LATIN_2		5
#define		TYPE_WINDOWS		6
#define		TYPE_MICROVEX		7
#define		TYPE_UNIA		8
#define		TYPE_MACINTOSH		9

#define		TYPE_CSK		10
#define		TYPE_CYFROMAT		11
#define		TYPE_DHN		12
#define		TYPE_IINTE		13
#define		TYPE_SWIERK		14
#define		TYPE_LOGIC		15
#define		TYPE_VENTURA		16
#define		TYPE_ELWRO		17
#define		TYPE_AMIGA		18
#define		TYPE_TEXPL		19
#define		TYPE_ATARI		20
#define		TYPE_COREL		21
#define		TYPE_ATM		22
#define		TYPE_BEZ		23

#define		TYPE_BRAIL		24
#define		TYPE_ISO_6937_2		25
#define		TYPE_UNICODE_UCS4	26
#define		TYPE_UNICODE_UCS2	27
#define		TYPE_UNICODE_UTF8	28
#define		TYPE_UNICODE_UTF7	29
#define		TYPE_UNICODE_UTF7_O	30	/* used in utf7.C */
#define		TYPE_UNICODE_JAVA	31
#define		TYPE_HEX		32
#define		TYPE_HTML		33
#define		TYPE_ISO_8859_1		34
#define		TYPE_OHIDA		35
#define		MAX_TRANS		36 /* Must be greater than any TYPE */

extern void init_test(		void );
extern void init_prefix( 	void );
extern void init_file(		void );
extern void init_iso_8859_2(	void );
extern void init_iso_8859_1(	void );
extern void init_mazowia(	void );
extern void init_latin_2(	void );
extern void init_windows(	void );
extern void init_microvex(	void );
extern void init_unia(		void );
extern void init_macintosh(	void );

extern void init_csk(		void );
extern void init_cyfromat(	void );
extern void init_dhn(		void );
extern void init_iinte(		void );
extern void init_swierk(	void );
extern void init_logic(		void );
extern void init_ventura(	void );
extern void init_elwro(		void );
extern void init_amiga(		void );
extern void init_texpl(		void );
extern void init_atari(		void );
extern void init_corel(		void );
extern void init_atm(		void );
extern void init_bez(		void );

extern void init_brail(		void );
extern void init_iso_6937_2(	void );
extern void init_unicode_ucs4(	void );
extern void init_unicode_ucs2(	void );
extern void init_unicode_utf8(	void );
extern void init_unicode_utf7(	void );
extern void init_unicode_java(	void );
extern void init_hex(		void );
extern void init_html(		void );
extern void init_ohida(		void );

extern void set_prefix( char prefix, char *name, int mode );
extern void set_file(   char *table, char *name, int mode );
extern void dump_short_table( void );
extern void dump_full_table( void );
/*********
extern void set_mazowia(	char *name );
extern void set_latin_2(	char *name );
extern void set_iso_8859_2(	char *name );
extern void set_iso_8859_1(	char *name );
extern void set_windows(	char *name );
extern void set_microvex(	char *name );
extern void set_unia(		char *name );
extern void set_iso_6937_2(	char *name );
extern void set_brail(		char *name );
extern void set_unicode_java(	char *name );
extern void set_macintosh(	char *name );
extern void set_test(		char *name );
********/
extern int  run_test( void );

/*===================================================================*/
typedef struct
  {
  char *key;
  char *long_key;
  char *name;
  void (*reset_inp)( void );
  void (*reset_out)( void );
  void (*encode)( void );
  void (*decode)( void );
  tbl_ptr inp_tbl;
  int  inp_tbl_size;
  tbl_ptr out_tbl;
  int  out_tbl_size;
  int  inp_ffree;
  int  inp_fused;
  void (*buf_tst)( void );
  int  (*freq_tst)( void );
  int  score;
  } trans_table;

extern trans_table trans[ MAX_TRANS ];
extern long int frequency[ 256 ];

/*********************************************************************/
