/*********************************************************************/
/* prefix.c     v 2.3      by J.Sobczyk            2001.08.02  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/* Prefix notation                                                   */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
code_tbl prefix_tbl = /* Latin 2 ISO 8859-2 */
  {
    { 'a',  0x0105 },   /* LATIN SMALL LETTER A WITH OGONEK */
    { 'c',  0x0107 },   /* LATIN SMALL LETTER C WITH ACUTE */
    { 'e',  0x0119 },   /* LATIN SMALL LETTER E WITH OGONEK */
    { 'l',  0x0142 },   /* LATIN SMALL LETTER L WITH STROKE */
    { 'n',  0x0144 },   /* LATIN SMALL LETTER N WITH ACUTE */
    { 'o',  0x00F3 },   /* LATIN SMALL LETTER O WITH ACUTE */
    { 's',  0x015B },   /* LATIN SMALL LETTER S WITH ACUTE */
    { 'x',  0x017A },   /* LATIN SMALL LETTER Z WITH ACUTE */
    { 'z',  0x017C },   /* LATIN SMALL LETTER Z WITH DOT ABOVE */
    { 'A',  0x0104 },   /* LATIN CAPITAL LETTER A WITH OGONEK */
    { 'C',  0x0106 },   /* LATIN CAPITAL LETTER C WITH ACUTE */
    { 'E',  0x0118 },   /* LATIN CAPITAL LETTER E WITH OGONEK */
    { 'L',  0x0141 },   /* LATIN CAPITAL LETTER L WITH STROKE */
    { 'N',  0x0143 },   /* LATIN CAPITAL LETTER N WITH ACUTE */
    { 'O',  0x00D3 },   /* LATIN CAPITAL LETTER O WITH ACUTE */
    { 'S',  0x015A },   /* LATIN CAPITAL LETTER S WITH ACUTE */
    { 'X',  0x0179 },   /* LATIN CAPITAL LETTER Z WITH ACUTE */
    { 'Z',  0x017B }    /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

long int pfreq[ 256 ];

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void prefix_decode( void )
  {
  int i;
  unicode ch;
 
  while ((inp_fused < inp_ffree) &&
         (uni_ffree < uni_size-1 ))
    {
    ch = (unicode) inp_buf[ inp_fused++ ];

    if (ch == inp_prefix)
      {
      ch = (unicode) inp_buf[ inp_fused++ ];
      if (ch != inp_prefix)
	{
	for (i=0; i<inp_tbl_size; i++)
	  if (ch == inp_tbl[i][0])
	    {
	    ch = inp_tbl[i][1];
	    break;
	    }
	if (i >= inp_tbl_size)
	  {
	  fprintf( stderr, "Invalid sequence: %c%c\a\n",
			inp_prefix, (char)ch );
	  uni_buf[ uni_ffree++ ] = inp_prefix;
	  }
        }
      }

    uni_buf[ uni_ffree++ ] = ch;
    }
 
  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void prefix_encode( void )
  {
  int i;
  unicode ch;
 
  while ((uni_ffree > uni_fused) &&
         (out_ffree < out_size-1))	/* Reserve one place in the  */
    {					/* buffer for the prefix     */
    ch = uni_buf[uni_fused++];
 
    for (i=0; i<out_tbl_size; i++)
      if (ch == out_tbl[i][1])
	{
	out_buf[out_ffree++] = (byte) out_prefix;
        ch = out_tbl[i][0];
	break;
	}
 
    out_buf[out_ffree++] = (byte) ch;
    }
 
  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_buf_prefix( void )
  {
  /****
  int i;

  for (i=0; i< 2*POL_NR; i++)
    if (buff[1] == prefix_tbl[i])
      pfreq[ buff[0] & 0xFF ]++;
  *******/
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_prefix( void )
  {
  int ch;
  int best;
  int best_ch;

  best    = -1;
  best_ch = -1;
  for (ch=0; ch<256; ch++)
    {
    if (isalpha(ch) || isspace(ch))
      {
      if ((verbose_flag > 2) && (pfreq[ch] != 0))
	printf( "Prefix[%02x]=%5ld\n", ch, pfreq[ch] );
      }
    else 
      {
      if (pfreq[ch] > best)
	{
	best_ch = ch;
	best    = pfreq[ch];
	}

      if ((verbose_flag > 1) && (pfreq[ch] != 0))
	printf( "Prefix[%02x]=%5ld\n", ch, pfreq[ch] );
      }
    }

  inp_prefix = best_ch;
  return best;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_prefix( char prefix, char *file, int mode )
  {
  switch (mode)
    {
    case 0:   inp_name		= file;
	      inp_type		= TYPE_PREFIX;
	      inp_prefix	= prefix;
	      inp_tbl		= prefix_tbl;
	      inp_tbl_size	= 2*POL_NR;
	      decode		= prefix_decode;
	      break;
    case 1:   out_name		= file;
	      out_type		= TYPE_PREFIX;
	      out_prefix	= prefix;
	      out_tbl		= prefix_tbl;
	      out_tbl_size	= 2*POL_NR;
	      encode		= prefix_encode;
	      break;
    default:  fprintf( stderr, "Too many switches\a\n" );
	      help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_prefix( void )
  {
  int i;

  for (i=0; i<256; i++)
    pfreq[i] = 0;

  trans[ TYPE_PREFIX ].key		= (char*)"-p@";
  trans[ TYPE_PREFIX ].name		= (char*)"Prefix notation";
  trans[ TYPE_PREFIX ].reset_inp	= NULL; /* Exception - called indirectly */
  trans[ TYPE_PREFIX ].reset_out	= NULL; /* Exception - called indirectly */
  trans[ TYPE_PREFIX ].encode		= prefix_encode;
  trans[ TYPE_PREFIX ].decode		= prefix_decode;
  trans[ TYPE_PREFIX ].inp_tbl		= prefix_tbl;
  trans[ TYPE_PREFIX ].inp_tbl_size	= 2*POL_NR;
  trans[ TYPE_PREFIX ].out_tbl		= prefix_tbl;
  trans[ TYPE_PREFIX ].out_tbl_size	= 2*POL_NR;
  trans[ TYPE_PREFIX ].buf_tst		= test_buf_prefix;
  trans[ TYPE_PREFIX ].freq_tst		= test_freq_prefix;
  }

/*********************************************************************/
