/*********************************************************************/
/* table.c      v 2.3      by J.Sobczyk            2001.08.02  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/* Table conversion procedures.                                      */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

code_tbl polchr = 
  {
    { 0,   0x0105 },   /* LATIN SMALL LETTER A WITH OGONEK */
    { 0,   0x0107 },   /* LATIN SMALL LETTER C WITH ACUTE */
    { 0,   0x0119 },   /* LATIN SMALL LETTER E WITH OGONEK */
    { 0,   0x0142 },   /* LATIN SMALL LETTER L WITH STROKE */
    { 0,   0x0144 },   /* LATIN SMALL LETTER N WITH ACUTE */
    { 0,   0x00F3 },   /* LATIN SMALL LETTER O WITH ACUTE */
    { 0,   0x015B },   /* LATIN SMALL LETTER S WITH ACUTE */
    { 0,   0x017A },   /* LATIN SMALL LETTER Z WITH ACUTE */
    { 0,   0x017C },   /* LATIN SMALL LETTER Z WITH DOT ABOVE */
    { 0,   0x0104 },   /* LATIN CAPITAL LETTER A WITH OGONEK */
    { 0,   0x0106 },   /* LATIN CAPITAL LETTER C WITH ACUTE */
    { 0,   0x0118 },   /* LATIN CAPITAL LETTER E WITH OGONEK */
    { 0,   0x0141 },   /* LATIN CAPITAL LETTER L WITH STROKE */
    { 0,   0x0143 },   /* LATIN CAPITAL LETTER N WITH ACUTE */
    { 0,   0x00D3 },   /* LATIN CAPITAL LETTER O WITH ACUTE */
    { 0,   0x015A },   /* LATIN CAPITAL LETTER S WITH ACUTE */
    { 0,   0x0179 },   /* LATIN CAPITAL LETTER Z WITH ACUTE */
    { 0,   0x017B }    /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void table_decode( void )
  {
  int i;
  unicode ch;

  while ((inp_fused < inp_ffree) &&
	 (uni_ffree < uni_size ))
    {
    ch = (unicode) inp_buf[ inp_fused++ ];
    for (i=0; i<inp_tbl_size; i++)
      if (ch == inp_tbl[i][0])
	{
        ch = inp_tbl[i][1];
	break;
	}
    uni_buf[ uni_ffree++ ] = ch;
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void table_encode( void )
  {
  int i;
  unicode ch;

  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size))
    {
    ch = uni_buf[uni_fused++];

    for (i=0; i<out_tbl_size; i++)
      if (ch == out_tbl[i][1])
	{
        ch = out_tbl[i][0];
	break;
	}

    out_buf[out_ffree++] = (byte) ch;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int table_test( tbl_ptr table, int table_size )
  {
  int i, j;
  int ch;
  int hits;
  int fails;

  for (i=0; i<2*POL_NR; i++)
    polchr[i][0] = polchr[i][1];

  for (j = 0; j<table_size; j++)
    for (i=0; i<2*POL_NR; i++)
      {
      if (table[j][1] == polchr[i][1])
	polchr[i][0] = table[j][0];
      }
  hits	= 0;
  fails	= 0;
  for (ch=128; ch<256; ch++)
    fails += frequency[ ch ];

  for (i=0; i< 2*POL_NR; i++)
    {
    hits  += frequency[ polchr[i][0] ];
    fails -= frequency[ polchr[i][0] ];
    }

/*  printf( "hits=%5d fails=%5d\n", hits, fails ); */

  return hits-fails;
  }

/*********************************************************************/
