/*********************************************************************/
/* test.c       v 2.3      by J.Sobczyk            2001.08.02  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/* Character code detector.                                          */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
code_tbl test_tbl = /* Latin 2 ISO 8859-2 */
  {
  #ifdef XXX
  { 0x0A1,  0x0104 }, /* LATIN CAPITAL LETTER A WITH OGONEK */
  { 0x0A3,  0x0141 }, /* LATIN CAPITAL LETTER L WITH STROKE */
  { 0x0A6,  0x015A }, /* LATIN CAPITAL LETTER S WITH ACUTE */
  { 0x0AC,  0x0179 }, /* LATIN CAPITAL LETTER Z WITH ACUTE */
  { 0x0AF,  0x017B }, /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  { 0x0B1,  0x0105 }, /* LATIN SMALL LETTER A WITH OGONEK */
  { 0x0B3,  0x0142 }, /* LATIN SMALL LETTER L WITH STROKE */
  { 0x0B6,  0x015B }, /* LATIN SMALL LETTER S WITH ACUTE */
  { 0x0BC,  0x017A }, /* LATIN SMALL LETTER Z WITH ACUTE */
  { 0x0BF,  0x017C }, /* LATIN SMALL LETTER Z WITH DOT ABOVE */
  { 0x0C6,  0x0106 }, /* LATIN CAPITAL LETTER C WITH ACUTE */
  { 0x0CA,  0x0118 }, /* LATIN CAPITAL LETTER E WITH OGONEK */
  { 0x0D1,  0x0143 }, /* LATIN CAPITAL LETTER N WITH ACUTE */
  { 0x0E6,  0x0107 }, /* LATIN SMALL LETTER C WITH ACUTE */
  { 0x0EA,  0x0119 }, /* LATIN SMALL LETTER E WITH OGONEK */
  { 0x0F1,  0x0144 }, /* LATIN SMALL LETTER N WITH ACUTE */
  { 0x0D3,  0x00D3 }, /* LATIN CAPITAL LETTER O WITH ACUTE */
  { 0x0F3,  0x00F3 }, /* LATIN SMALL LETTER O WITH ACUTE */
  #endif


    { '\xB1',  'a' },
    { '\xE6',  'c' },
    { '\xEA',  'e' },
    { '\xB3',  'l' },
    { '\xF1',  'n' },
    { '\xF3',  'o' },
    { '\xB6',  's' },
    { '\xBC',  'x' },
    { '\xBF',  'z' },
    { '\xA1',  'A' },
    { '\xC6',  'C' },
    { '\xCA',  'E' },
    { '\xA3',  'L' },
    { '\xD1',  'N' },
    { '\xD3',  'O' },
    { '\xA6',  'S' },
    { '\xAC',  'X' },
    { '\xAF',  'Z' }
  };

long int frequency[ 256 ];

unicode w_az[] =	{0x61, 0x17C, 0};				/* a */
unicode w_byla[] =	{0x62, 0x79, 0x142, 0x61, 0};			/* bya */
unicode w_bylo[] =	{0x62, 0x79, 0x142, 0x6F, 0};			/* byo */
unicode w_byc[] =	{0x62, 0x79, 0x107, 0};				/* by */
unicode w_bedzie[] =	{0x62, 0x119, 0x64, 0x7A, 0x69, 0x65, 0};	/* bdzie */
unicode w_beda[] =	{0x62, 0x119, 0x64, 0x105, 0};			/* bd */
unicode w_duzy[] =	{0x64, 0x75, 0x17C, 0x79, 0};			/* duy */
unicode w_gdyz[] =	{0x67, 0x64, 0x79, 0x17C, 0};			/* gdy */
unicode w_ilosc[] =	{0x69, 0x6C, 0x6F, 0x15B, 0x107, 0};		/* ilo */
unicode w_imie[] =	{0x69, 0x6D, 0x69, 0x119, 0};			/* imi */
unicode w_jesli[] =	{0x6A, 0x65, 0x15B, 0x6C, 0x69, 0};		/* jeli */
unicode w_juz[] =	{0x6A, 0x75, 0x17C, 0};				/* ju */
unicode w_kazdy[] =	{0x6B, 0x61, 0x17C, 0x64, 0x79, 0};		/* kady */
unicode w_kazdym[] =	{0x6B, 0x61, 0x17C, 0x64, 0x79, 0x6D, 0};	/* kadym */
unicode w_ktore[] =	{0x6B, 0x74, 0xF3, 0x72, 0x65, 0};		/* ktre */
unicode w_ktorych[] =	{0x6B, 0x74, 0xF3, 0x72, 0x79, 0x63, 0x68, 0};	/* ktrych */
unicode w_maja[] =	{0x6D, 0x61, 0x6A, 0x105, 0};			/* maj */
unicode w_moga[] =	{0x6D, 0x6F, 0x67, 0x105, 0};			/* mog */
unicode w_moze[] =	{0x6D, 0x6F, 0x17C, 0x65, 0};			/* moe */
unicode w_mozliwe[] =	{0x6D, 0x6F, 0x17C, 0x6C, 0x69, 0x77, 0x65, 0};	/* moliwe */
unicode w_mozna[] =	{0x6D, 0x6F, 0x17C, 0x6E, 0x61, 0};		/* mona */
unicode w_nalezy[] =	{0x6E, 0x61, 0x6C, 0x65, 0x17C, 0x79, 0};	/* naley */
unicode w_niz[] =	{0x6E, 0x69, 0x17C, 0};				/* ni */
unicode w_osob[] =	{0x6F, 0x73, 0xF3, 0x62, 0};			/* osb */
unicode w_pomiedzy[] =	{0x70, 0x6F, 0x6D, 0x69, 0x119, 0x64, 0x7A, 0x79, 0};	/* pomidzy */
unicode w_poniewaz[] =	{0x70, 0x6F, 0x6E, 0x69, 0x65, 0x77, 0x61, 0x17C, 0};	/* poniewa */
unicode w_rzad[] =	{0x72, 0x7A, 0x105, 0x64, 0};			/* rzd */
unicode w_rowniez[] =	{0x72, 0xF3, 0x77, 0x6E, 0x69, 0x65, 0x17C, 0};	/* rwnie */
unicode w_roznych[] =	{0x72, 0xF3, 0x17C, 0x6E, 0x79, 0x63, 0x68, 0};	/* rnych */
unicode w_sie[] =	{0x73, 0x69, 0x119, 0};				/* si */
unicode w_soba[] =	{0x73, 0x6F, 0x62, 0x105, 0};			/* sob */
unicode w_sa[] =	{0x73, 0x105, 0};				/* s */
unicode w_tez[] =	{0x74, 0x65, 0x17C, 0};				/* te */
unicode w_uwage[] =	{0x75, 0x77, 0x61, 0x67, 0x119, 0};		/* uwag */
unicode w_wiec[] =	{0x77, 0x69, 0x119, 0x63, 0};			/* wic */
unicode w_wieksza[] =	{0x77, 0x69, 0x119, 0x6B, 0x73, 0x7A, 0x61, 0};	/* wiksza */
unicode w_wzdluz[] =	{0x77, 0x7A, 0x64, 0x142, 0x75, 0x17C, 0};	/* wzdu */
unicode w_ze[] =	{0x17C, 0x65, 0};				/* e */
unicode w_ksiaz[] =	{0x6B, 0x73, 0x69, 0x105, 0x17C, 0};		/* ksi ?? */
unicode w_osci[] =	{0x6F, 0x15B, 0x63, 0x69, 0};			/* oci ?? */
unicode w_osc[] =	{0x6F, 0x15B, 0x107, 0};			/* o ?? */
unicode w_ow[] =	{0xF3, 0x77, 0};				/* w ?? */

unicode *word[] = 
  {
  w_az, w_byla, w_bylo, w_byc, w_bedzie, w_beda, w_duzy, w_gdyz, w_ilosc, w_imie,
  w_jesli, w_juz, w_kazdy, w_kazdym, w_ktore, w_ktorych, w_maja, w_moga, w_moze,
  w_mozliwe, w_mozna, w_nalezy, w_niz, w_osob, w_pomiedzy, w_poniewaz, w_rzad,
  w_rowniez, w_roznych, w_sie, w_soba, w_sa, w_tez, w_uwage, w_wiec, w_wieksza,
  w_wzdluz, w_ze, w_ksiaz, w_osci, w_osc, w_ow,
  NULL   
  };

int wscores[] = 
  {
    2,	/* a */
    4,	/* bya */
    4,	/* byo */
    3,	/* by */
    6,	/* bdzie */
    4,	/* bd */
    4,	/* duy */
    4,	/* gdy */
    5,	/* ilo */
    4,	/* imi */
    5,	/* jeli */
    3,	/* ju */
    5,	/* kady */
    6,	/* kadym */
    5,	/* ktre */
    7,	/* ktrych */
    4,	/* maj */
    4,	/* mog */
    4,	/* moe */
    7,	/* moliwe */
    5,	/* mona */
    6,	/* naley */
    3,	/* ni */
    4,	/* osb */
    8,	/* pomidzy */
    8,	/* poniewa */
    4,	/* rzd */
    7,	/* rwnie */
    7,	/* rnych */
    3,	/* si */
    4,	/* sob */
    2,	/* s */
    3,	/* te */
    5,	/* uwag */
    4,	/* wic */
    7,	/* wiksza */
    6,	/* wzdu */
    2,	/* e */
    2,	/* ksi ?? */
    2,	/* oci ?? */
    2,	/* o ?? */
    2,	/* w ?? */
    -1
  };

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_decode( void )
  {
  fprintf( stderr, "Test failed\n" );
  exit( 1 );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_encode( void )
  {
  fprintf( stderr, "Test failed\n" );
  exit( 1 );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int str_len( unicode *str )
  {
  int i;
  for (i=0; str[i]!=0; i++)
    ;
  return i;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int search_str( unicode *str, int start, int stop )
  {
  int i, k, l;

  stop-=str_len(str);
  l=str_len(str);
  for (i=start; i<stop; i++)
    {
    for (k=0; str[k]!=0; k++)
      if (uni_buf[i+k] != str[k])
	break;
    if (str[k] == 0)
      return i;
    }
  return -1;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void run_buf_tests( int tr )
  {
  int i, n;

  for (i=0; word[i]!=NULL; i++)
    {
    for (n=uni_fused; n<uni_ffree; n++)
      {
      n = search_str( word[i], n, uni_ffree );
      if (n < 0)
	break;
      else
	trans[tr].score += wscores[i];
      }
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void scan_file( void )
  {
  int ch;
  int i;
  int tst_inp_shift = 0;
  int tst_inp_fused = 0;
  int min_inp_fused = 0;
  unsigned long offset = 0;

  if (!strcmp(inp_name, "-"))
    inp_file = stdin;
  else
    {
    inp_file = fopen( inp_name, "r" );
    if (inp_file == NULL)
      {
      fprintf( stderr, "Can not open input file: %s\a\n", inp_name );
      exit( 2 );
      }
    }

  for (ch=0; ch <256; ch++)
    frequency[ch] = 0;

  for (i = 0; i<MAX_TRANS; i++)
    {
    trans[ i ].inp_ffree = 0;
    trans[ i ].inp_fused = 0;
    trans[ i ].score     = 0;
    }

  if (verbose_flag > 2)
    printf( "Scanning file ...\n" );

  while (!inp_eof)
    {
    shift();
    readfile();
    tst_inp_fused = inp_fused;
    tst_inp_shift = min_inp_fused - tst_inp_fused;

    for (i=min_inp_fused; i<inp_ffree; i++)
      frequency[inp_buf[i]]++;

    min_inp_fused = inp_ffree;
    for (i = 0; i<MAX_TRANS; i++)
      {
      if (i == TYPE_TEST || i == TYPE_FILE || i == TYPE_PREFIX || i == TYPE_HEX)
	continue;
      inp_fused = tst_inp_fused + trans[i].inp_fused - tst_inp_shift;
      uni_ffree = 0;
      uni_fused = 0;

      inp_tbl           = trans[ i ].inp_tbl;
      inp_tbl_size      = trans[ i ].inp_tbl_size;

      trans[i].decode();

      run_buf_tests(i);

      trans[i].inp_fused = inp_fused;
      if (inp_fused < min_inp_fused)
	min_inp_fused = inp_fused;
      }
    
    inp_fused = min_inp_fused;
    offset += min_inp_fused;
    }

  fclose( inp_file );

  if (verbose_flag > 2)
    printf( "Scanning file DONE %lu\n", offset );

  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int find_best_code( void )
  {
  int t;
  int best;
  int best_t;
  int res;
  int score;
  int freq;

  best   = -1;
  best_t = -1;
  for (t=0; t<MAX_TRANS; t++)
    {
    if (trans[ t ].freq_tst != NULL)
      {
/*      printf( "Testing %s\n", trans[ t ].name );*/
      /*--- Calculate frequency ----*/
      freq = 0;
      if (trans[ t ].freq_tst != NULL)
	freq = (*trans[ t ].freq_tst)();
      /*--- Get word score ----*/
      score = trans[ t ].score;
      res = freq + 10 * score;
      if (res > best)
	{
	best_t = t;
	best   = res;
	}
      if (((verbose_flag > 0) && (res > 0)) ||
           (verbose_flag > 2) )
	printf( "Test %-40s OK (quality=%d,words=%d,freq=%d)\n",
		      trans[ t ].name, res, score, freq );
      }

    }

  if (best == -1)
    best_t = TYPE_ISO_8859_2;	/* Fallback code */

  if (best_t == TYPE_PREFIX)
    {
    inp_type = TYPE_UNSPECIFIED;
    set_prefix( inp_prefix, inp_name, 0 );
    if ((verbose_flag > 0) || (out_name == NULL))
      printf( "Best code is: %s using %c (quality=%d)\n",
			     trans[ best_t ].name, inp_prefix, best );
    }
  else if (best_t >= 0)
    {
    inp_type = TYPE_UNSPECIFIED;
/*    if (trans[best_t].set != NULL)*/
/*      (*trans[best_t].set)( inp_name );*/
    if ((verbose_flag > 0) || (out_name == NULL))
      printf( "Best code is: %s (quality=%d)\n",
			     trans[ best_t ].name, best );
    }

  return best_t;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int  run_test( void )
  {
  int ch;

  scan_file();

  if (verbose_flag > 2)
    for (ch=0; ch <128; ch++)
      {
      if (frequency[ch] != 0)
	printf( "Frequency[%02x]=%5ld\n", ch, frequency[ch] );
      }

  if (verbose_flag > 1)
    for (ch=128; ch <256; ch++)
      {
      if (frequency[ch] != 0)
	printf( "Frequency[%02x]=%5ld\n", ch, frequency[ch] );
      }

  return find_best_code();
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_test( void )
  {
  trans[ TYPE_TEST ].key		= (char*)"-t";
  trans[ TYPE_TEST ].long_key		= (char*)"-test";
  trans[ TYPE_TEST ].name		= (char*)"Test";
  trans[ TYPE_TEST ].reset_inp		= NULL;
  trans[ TYPE_TEST ].reset_out		= NULL;
  trans[ TYPE_TEST ].decode		= test_decode;
  trans[ TYPE_TEST ].encode		= test_encode;
  trans[ TYPE_TEST ].inp_tbl		= test_tbl;
  trans[ TYPE_TEST ].inp_tbl_size	= 2*POL_NR;
  trans[ TYPE_TEST ].out_tbl		= test_tbl;
  trans[ TYPE_TEST ].out_tbl_size	= 2*POL_NR;
  trans[ TYPE_TEST ].buf_tst		= NULL;
  trans[ TYPE_TEST ].freq_tst		= NULL;
  }

/*********************************************************************/
