/*********************************************************************/
/* ucs4.c    v 2.3      by J.Sobczyk            2001.08.02  17:01    */
/*                                                                   */
/* Polish document converter.                                        */
/* ISO 10646 UCS-4 with additions defined in RFC 2070                */
/*********************************************************************/
/* POLCNV -  a program for converting text documents between         */
/* different character encoding methods and stadards.                */
/* Supports almost all codes historically used in Poland             */
/* but is not limited to them.                                       */
/*                                                                   */
/* Copyright (C) 1994-2003  Jerzy Sobczyk                            */
/*                                                                   */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License       */
/* as published by the Free Software Foundation; either version 2    */
/* of the License, or (at your option) any later version.            */
/*                                                                   */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/* GNU General Public License for more details.                      */
/*                                                                   */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place - Suite 330,                    */
/* Boston, MA  02111-1307, USA.                                      */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

long int ucs4_hits  	= 0;
long int ucs4_fails 	= 0;
int ucs4_0nbsp		= TRUE;
int ucs4_first		= TRUE;
int ucs4_reverse	= FALSE;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs4_decode( void )
  {
  int limit;

  limit = inp_ffree;
  if (!inp_eof)
    limit -= (limit - inp_fused) % 4;
  else if ((limit - inp_fused) % 4 != 0)
    {
    limit -= (limit - inp_fused) % 4;
    fprintf( stderr, "Uncomplete character " );
    while ((limit-inp_fused)%4 != 0)
      fprintf( stderr, "%02x", inp_buf[ limit++ ] );
    fprintf( stderr, " at the end of file - padded with 0's\n" );
    }

  if (ucs4_first)
    {
    if (      inp_buf[inp_fused  ] == 0x0FF && 
	      inp_buf[inp_fused+1] == 0x0FE && 
	      inp_buf[inp_fused+2] == 0x000 && 
	      inp_buf[inp_fused+3] == 0x000    )
      {
      fprintf( stderr, "Low-high byte order detected - reversing!\n" );
      inp_fused += 4;
      ucs4_reverse = TRUE;
      }
    else if ( inp_buf[inp_fused  ] == 0x000 && 
	      inp_buf[inp_fused+1] == 0x000 && 
	      inp_buf[inp_fused+2] == 0x0FE && 
	      inp_buf[inp_fused+3] == 0x0FF    )
      {
      inp_fused += 4;
      }
    ucs4_first = FALSE;
    }

  while ((inp_fused < limit) &&
	 (uni_ffree < uni_size ))
    if (!ucs4_reverse)
      {
      uni_buf[ uni_ffree++ ] =
	((unicode)(inp_buf[ inp_fused   ]) << 24) +
	((unicode)(inp_buf[ inp_fused+1 ]) << 16) +
	((unicode)(inp_buf[ inp_fused+2 ]) << 8) +
	 (unicode)(inp_buf[ inp_fused+3 ]);
      inp_fused += 4;
      }
    else	/* Reversed byte odrer				     */
      {
      uni_buf[ uni_ffree++ ] =
	 (unicode)(inp_buf[ inp_fused   ]) +
	((unicode)(inp_buf[ inp_fused+1 ]) << 8) +
	((unicode)(inp_buf[ inp_fused+2 ]) << 16) +
	((unicode)(inp_buf[ inp_fused+3 ]) << 24);
      inp_fused += 4;
      }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs4_encode( void )
  {
  if (ucs4_0nbsp)
    {
    if (uni_buf[uni_fused] != 0x0FEFFUL)
      {		/* Always start with ZERO-WIDTH NON-BREAKING SPACE   */
		/* as recommended in RFC 2070			     */
      out_buf[out_ffree++] = 0x000;
      out_buf[out_ffree++] = 0x000;
      out_buf[out_ffree++] = 0x0FE;
      out_buf[out_ffree++] = 0x0FF;
      }
    ucs4_0nbsp = FALSE;
    }

  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size-3))
    {
    out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 24) & 0x0FF);
    out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 16) & 0x0FF);
    out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 8)  & 0x0FF);
    out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x0FF);

    uni_fused++;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_ucs4( byte *buff )
  {
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_ucs4( void )
  {
  return ucs4_hits-ucs4_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs4_reset_inp( void )
  {
  ucs4_first	= TRUE;
  ucs4_reverse	= FALSE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs4_reset_out( void )
  {
  ucs4_0nbsp	= TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_unicode_ucs4( void )
  {
  trans[ TYPE_UNICODE_UCS4 ].key		= (char*)"-u4";
  trans[ TYPE_UNICODE_UCS4 ].long_key		= (char*)"-ucs-4";
  trans[ TYPE_UNICODE_UCS4 ].name		= (char*)"Unicode UCS-4";
  trans[ TYPE_UNICODE_UCS4 ].reset_inp		= ucs4_reset_inp;
  trans[ TYPE_UNICODE_UCS4 ].reset_out		= ucs4_reset_out;
  trans[ TYPE_UNICODE_UCS4 ].decode		= ucs4_decode;
  trans[ TYPE_UNICODE_UCS4 ].encode		= ucs4_encode;
  trans[ TYPE_UNICODE_UCS4 ].inp_tbl		= NULL;
  trans[ TYPE_UNICODE_UCS4 ].inp_tbl_size	= 0;
  trans[ TYPE_UNICODE_UCS4 ].out_tbl		= NULL;
  trans[ TYPE_UNICODE_UCS4 ].out_tbl_size	= 0;
  trans[ TYPE_UNICODE_UCS4 ].buf_tst		= NULL;
  trans[ TYPE_UNICODE_UCS4 ].freq_tst		= test_freq_ucs4;
  }

/*********************************************************************/
